@@ -25,13 +25,12 @@
 #include <stdio.h>
 #include <sys/wait.h>
 
-#include <gnome.h>
-#include <eel/eel.h>
 #include <libgnomevfs/gnome-vfs.h>
 
 #include "seahorse-gpgmex.h"
 #include "seahorse-util.h"
 #include "seahorse-key.h"
+#include "seahorse-gconf.h"
 #include "seahorse-vfs-data.h"
 
 typedef struct _SeahorsePGPHeader {
@@ -214,14 +213,16 @@ seahorse_util_gpgme_to_error(gpgme_error_t gerr, GError** err)
  * seahorse_util_write_data_to_file:
  * @path: Path of file to write to
  * @data: Data to write
+ * @release: Whether to release @data or not
  *
- * Tries to write @data to the file @path. @data will be released upon completion.
+ * Tries to write @data to the file @path. 
  *
  * Returns: GPG_ERR_NO_ERROR if write is successful,
  * GPGME_File_Error if the file could not be opened
  **/
 gpgme_error_t
-seahorse_util_write_data_to_file (const gchar *path, gpgme_data_t data)
+seahorse_util_write_data_to_file (const gchar *path, gpgme_data_t data, 
+                                  gboolean release)
 {
     gpgme_error_t err = GPG_OK;
 	gpgme_data_t file;
@@ -238,37 +239,39 @@ seahorse_util_write_data_to_file (const gchar *path, gpgme_data_t data)
     if (file != NULL) {
     	buffer = g_new0 (gchar, 128);
     	
-    	while ((nread = gpgme_data_read (data, buffer, 128)) > 0)
-        {
-            if(gpgme_data_write (file, buffer, nread) < 0)
-            {
+    	while ((nread = gpgme_data_read (data, buffer, 128)) > 0) {
+            if(gpgme_data_write (file, buffer, nread) < 0) {
                 gpg_err_code_t e = gpg_err_code_from_errno (errno);
                 err = GPG_E (e);                
                 break;
             }
         }
+
+        g_free (buffer);
     }
-	
-    gpgme_data_release (file);
-	gpgme_data_release (data);
-	
+
+    if (release)
+        gpgme_data_release (data);
+    
+    gpgme_data_release (file);	
 	return err;
 }
 
 /**
  * seahorse_util_write_data_to_text:
  * @data: Data to write
+ * @release: Whether to release @data or not
  *
- * Converts @data to a string. @data will be released upon completion.
+ * Converts @data to a string. 
  *
  * Returns: The string read from data
  **/
 gchar*
-seahorse_util_write_data_to_text (gpgme_data_t data)
+seahorse_util_write_data_to_text (gpgme_data_t data, gboolean release)
 {
 	gint size = 128;
-        gchar *buffer, *text;
-        guint nread = 0;
+    gchar *buffer, *text;
+    guint nread = 0;
 	GString *string;
 
     /* 
@@ -282,14 +285,71 @@ seahorse_util_write_data_to_text (gpgme_data_t data)
 	
 	while ((nread = gpgme_data_read (data, buffer, size)) > 0)
 		string = g_string_append_len (string, buffer, nread);
-	
-	gpgme_data_release (data);
+
+    if (release)
+        gpgme_data_release (data);
+    
 	text = string->str;
 	g_string_free (string, FALSE);
 	
 	return text;
 }
 
+/** 
+ * seahorse_util_read_data_block
+ * 
+ * Breaks out one block of data (usually a key)
+ * 
+ * @buf: A string buffer to write the data to.
+ * @data: The GPGME data block to read from.
+ * @start: The start signature to look for.
+ * @end: The end signature to look for.
+ * 
+ * Returns: The number of bytes copied.
+ */
+guint
+seahorse_util_read_data_block (GString *buf, gpgme_data_t data, 
+                               const gchar *start, const gchar* end)
+{
+    const gchar *t;
+    guint copied = 0;
+    gchar ch;
+     
+    /* Look for the beginning */
+    t = start;
+    while (gpgme_data_read (data, &ch, 1) == 1) {
+        
+        /* Match next char */            
+        if (*t == ch)
+            t++;
+
+        /* Did we find the whole string? */
+        if (!*t) {
+            buf = g_string_append (buf, start);
+            copied += strlen (start);
+            break;
+        }
+    } 
+    
+    /* Look for the end */
+    t = end;
+    while (gpgme_data_read (data, &ch, 1) == 1) {
+        
+        /* Match next char */
+        if (*t == ch)
+            t++;
+        
+        buf = g_string_append_c (buf, ch);
+        copied++;
+                
+        /* Did we find the whole string? */
+        if (!*t)
+            break;
+    }
+    
+    return copied;
+}
+
 /** 
  * seahorse_util_printf_fd:
  * @fd: The file descriptor to write to
@@ -685,6 +745,8 @@ seahorse_util_uris_package (const gchar* package, const char** uris)
     gchar *cmd;
     gchar *t;
     gchar *x;
+    GnomeVFSFileInfo *info;
+    GnomeVFSResult result;
     
     t = gnome_vfs_get_local_path_from_uri (package);
     x = g_shell_quote(t);
@@ -718,23 +780,32 @@ seahorse_util_uris_package (const gchar* package, const char** uris)
     r = g_spawn_command_line_sync (cmd, &out, NULL, &status, &err);
     g_free (cmd); 
     
-    if(out)
-    {
-        g_print(out);
-        g_free(out);
+    if (out) {
+        g_print (out);
+        g_free (out);
     }
     
     if (!r) {
-        seahorse_util_handle_error(err, _("Couldn't run file-roller"));
-        return FALSE;   
-    }
-    
-    if(!(WIFEXITED(status) && WEXITSTATUS(status) == 0)) {
-        seahorse_util_show_error(NULL, _("The file-roller process did not complete successfully"));
+        seahorse_util_handle_error (err, _("Couldn't run file-roller"));
         return FALSE;
     }
     
-    return TRUE;        
+    if (!(WIFEXITED (status) && WEXITSTATUS (status) == 0)) {
+        seahorse_util_show_error (NULL, _("The file-roller process did not complete successfully"));
+        return FALSE;
+    }
+    
+    info = gnome_vfs_file_info_new ();
+	info->permissions = GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE;
+	result = gnome_vfs_set_file_info (package, info, GNOME_VFS_SET_FILE_INFO_PERMISSIONS);
+    gnome_vfs_file_info_unref (info);
+	    
+	if (result != GNOME_VFS_OK) {
+    	seahorse_util_handle_error (err, _("Couldn't set permissions on backup file."));
+        return FALSE;
+    }
+
+    return TRUE;
 }
 
 GtkWidget*
@@ -772,8 +843,12 @@ void
 seahorse_util_chooser_show_key_files (GtkWidget *dialog)
 {
     GtkFileFilter* filter = gtk_file_filter_new ();
+    
+    /* Filter for PGP keys. We also include *.asc, as in many 
+       cases that extension is associated with text/plain */
     gtk_file_filter_set_name (filter, _("All key files"));
-    gtk_file_filter_add_mime_type (filter, "application/pgp-keys");    
+    gtk_file_filter_add_mime_type (filter, "application/pgp-keys");
+    gtk_file_filter_add_pattern (filter, "*.asc");    
     gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);    
     gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
 
@@ -931,7 +1006,7 @@ seahorse_util_add_suffix (gpgme_ctx_t ctx, const gchar *path,
     gchar *t;
 	
 	if (suffix == SEAHORSE_CRYPT_SUFFIX)
-        ext = eel_gconf_get_boolean (ARMOR_KEY) ? SEAHORSE_EXT_ASC : SEAHORSE_EXT_PGP;
+        ext = seahorse_gconf_get_boolean (ARMOR_KEY) ? SEAHORSE_EXT_ASC : SEAHORSE_EXT_PGP;
 	else if (suffix == SEAHORSE_ASC_SUFFIX)
 		ext = SEAHORSE_EXT_ASC;
 	else
