@@ -47,4 +47,14 @@ struct _SeahorseKeyManagerStoreClass
 SeahorseKeyStore*	seahorse_key_manager_store_new	(SeahorseKeySource	*sksrc,
 							                         GtkTreeView        *view);
 
+/* Drag and trop target types */
+enum SeahorseTargetTypes {
+    TEXT_PLAIN,
+    TEXT_URIS
+};
+
+/* Drag and drop targent entries */
+extern const GtkTargetEntry seahorse_target_entries[];
+extern guint seahorse_n_targets;
+
 #endif /* __SEAHORSE_KEY_MANAGER_STORE_H__ */
